ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
c                                                                     c
c This program takes twp cylindrical grid files, a u componend wind   c
c grid file and a v component wind grid file, and creates two new     c
c cylindrical grid files containing the tangential and radial wind    c
c components.  Either of these files can then be averaged azimuthally c
c and displayed in grads using the cyl2aza UDF.                       c
c                                                                     c
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      PROGRAM uv2tr

      IMPLICIT NONE

      INTEGER, PARAMETER :: iunit_gr_out=21, iunit_gr_in=22,
     & iunit_gr_u=23, iunit_gr_v=24, iunit_gr_t=25, iunit_gr_r=26

      REAL ovals(20), ivals(20), cyl_header(10)
      REAL, ALLOCATABLE :: grid_data(:)
      REAL, ALLOCATABLE :: u_data(:,:,:), v_data(:,:,:)
      REAL, ALLOCATABLE :: t_data(:,:,:), r_data(:,:,:)
      INTEGER nxdim, nydim, nsiz
      REAL lato, lono, startaz, endaz, range
      INTEGER nazimuth, nrange, nlevs
      INTEGER i, flag


c Open the files

      CALL open_files(iunit_gr_out,iunit_gr_in,
     &                iunit_gr_u,iunit_gr_v,
     &                iunit_gr_t,iunit_gr_r)

c  Load the grid header information from the grads output file

      READ(iunit_gr_out) ovals

c  Loat the grid header information from the cylindrical grid files
      
      READ(iunit_gr_u) cyl_header
      READ(iunit_gr_v) cyl_header
      
      lato     = cyl_header(1)
      lono     = cyl_header(2)
      nazimuth = nint(cyl_header(3))
      nrange   = nint(cyl_header(4))
      nlevs    = nint(cyl_header(5))
      startaz  = cyl_header(6)
      endaz    = cyl_header(7)
      range    = cyl_header(8)
      
c  Allocate appropriatly sized grids for the cylindrical grid data
c  and read in data

      ALLOCATE(u_data(nazimuth,nrange,nlevs))
      ALLOCATE(v_data(nazimuth,nrange,nlevs))
      ALLOCATE(t_data(nazimuth,nrange,nlevs))
      ALLOCATE(r_data(nazimuth,nrange,nlevs))

      READ(iunit_gr_u) u_data
      READ(iunit_gr_v) v_data

c  Do calculations on grids in momory

      CALL calc_tr_winds(u_data,v_data,nazimuth,nrange,nlevs,
     &                   lato,lono,startaz,endaz,range,
     &                   t_data,r_data)

c  Write results out to files

      WRITE(iunit_gr_t) cyl_header
      WRITE(iunit_gr_t) t_data
      WRITE(iunit_gr_r) cyl_header
      WRITE(iunit_gr_r) r_data

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  This section just prepares dummy data to write to the grads
c  input file.

c  Get the essential dimension informatio from the header and allocate
c  memory for the grid data

      nxdim = nint(ovals(4))
      nydim = nint(ovals(5))
      nsiz  = nxdim * nydim

      ALLOCATE(grid_data(nsiz))

c  Store grid data into memory

      READ(iunit_gr_out) grid_data

c  Copy grid header from grads output to grid header for GrADS input

      DO i=1,20
        ivals(i) = ovals(i)
      ENDDO

c  Write out dummy data to function result file

      CALL write_frf(iunit_gr_in,ivals,grid_data,nsiz)

      DEALLOCATE(grid_data)

c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c  Deallocata all arrays and close all open files

      DEALLOCATE(u_data)
      DEALLOCATE(v_data)
      DEALLOCATE(t_data)
      DEALLOCATE(r_data)

      CALL close_files(iunit_gr_out,iunit_gr_in,
     &                 iunit_gr_u,iunit_gr_v,
     &                 iunit_gr_t,iunit_gr_r)

      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      SUBROUTINE open_files(iunit_gr_out,iunit_gr_in,
     &                      iunit_gr_u,iunit_gr_v,
     &                      iunit_gr_t,iunit_gr_r)

      IMPLICIT NONE

      INTEGER iunit_gr_out, iunit_gr_in
      INTEGER iunit_gr_u, iunit_gr_v, iunit_gr_t, iunit_gr_r
      INTEGER nargs, dtffrmt
      REAL vals(20), uhead(10), vhead(10)
      CHARACTER(LEN=32) ufile, vfile, tfile, rfile
      CHARACTER(LEN=41) dirfile
      INTEGER i, flag

c  Open data transfer files

      OPEN (iunit_gr_out,FILE='./udfs/udf.uv2tr.out',
     &      STATUS='unknown', FORM='unformatted',ACCESS='sequential',
     &      POSITION='rewind',ERR=910)
           
      OPEN (iunit_gr_in,FILE='./udfs/udf.uv2tr.in',STATUS='unknown',
     &      FORM='unformatted',ACCESS='sequential',ERR=920)

      GOTO 400

  910 PRINT *, '!!! ERROR: unable to open udf.uv2tr.out'
      STOP 50

  920 PRINT *, '!!! ERROR: unable to open udf.uv2tr.in'
      STOP 50

  400 CONTINUE

c  Read data transfer file header

      READ(iunit_gr_out) vals

c  Make sure number of arguments is 5

      nargs = nint(vals(1))
      IF (nargs /= 5) THEN
        PRINT *, '!!! ERROR: incorrect number of arguments'
        STOP 50
      ENDIF
      
c  Check to make sure the data transfer file is the correct format

      dtffrmt = nint(vals(2))
      IF (dtffrmt /= 0) THEN
        PRINT *, '!!! ERROR: incorrect file transfer format'
	STOP 50
      ENDIF

c  Read the name of the u data grid file

      READ(iunit_gr_out) ufile
      dirfile = './udfs/'//ufile

c  Open the specified file

      OPEN (iunit_gr_u,FILE=dirfile,STATUS='old',
     &      FORM='unformatted',ACCESS='sequential',
     &      POSITION='rewind',ERR=930)

c  Read the name of the v data grid file

      READ(iunit_gr_out) vfile
      dirfile = './udfs/'//vfile

c  Open the specified file

      OPEN (iunit_gr_v,FILE=dirfile,STATUS='old',
     &      FORM='unformatted',ACCESS='sequential',
     &      POSITION='rewind',ERR=940)

c  Read the name of the t data grid file

      READ(iunit_gr_out) tfile
      dirfile = './udfs/'//tfile

c  Open the specified file

      OPEN (iunit_gr_t,FILE=dirfile,STATUS='replace',
     &      FORM='unformatted',ACCESS='sequential',
     &      POSITION='rewind',ERR=950)

c  Read the name of the r data grid file

      READ(iunit_gr_out) rfile
      dirfile = './udfs/'//rfile

c  Open the specified file

      OPEN (iunit_gr_r,FILE=dirfile,STATUS='replace',
     &      FORM='unformatted',ACCESS='sequential',
     &      POSITION='rewind',ERR=960)

      GOTO 500

930   PRINT *, '!!! ERROR: unable to open '//ufile
      STOP 50
940   PRINT *, '!!! ERROR: unable to open '//vfile
      STOP 50
950   PRINT *, '!!! ERROR: unable to open '//tfile
      STOP 50
960   PRINT *, '!!! ERROR: unable to open '//rfile
      STOP 50

  500 CONTINUE

c  Now make sure the u grid and v grid have the same header to ensure
c  that the dimensions are the same

      READ(iunit_gr_u) uhead
      REWIND(iunit_gr_u)
      READ(iunit_gr_v) vhead
      REWIND(iunit_gr_v)

      flag = 0
      DO i=1,10
        IF (uhead(i) /= vhead(i)) THEN
          flag = 1
        ENDIF
      ENDDO

      IF (flag) THEN
        PRINT *, '!!! ERROR: u and v grids do not have same header'
      ENDIF
      

      RETURN
      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      SUBROUTINE write_frf(iunit_gr_in,ivals,grid_data,nsiz)

c  This just writes dummy data to the function result file

      IMPLICIT NONE

      INTEGER iunit_gr_in
      INTEGER nsiz
      REAL vals(20), ivals(20), grid_data(nsiz)

      vals(1) = 0.0
      vals(2) = 0.0

      WRITE(iunit_gr_in) vals
      WRITE(iunit_gr_in) ivals
      WRITE(iunit_gr_in) grid_data

      RETURN
      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      SUBROUTINE calc_tr_winds(u_data,v_data,nazimuth,nrange,nlevs,
     &                         lato,lono,startaz,endaz,range,
     &                         t_data,r_data)

      IMPLICIT NONE

      INTEGER nazimuth, nrange, nlevs
      REAL u_data(nazimuth,nrange,nlevs), v_data(nazimuth,nrange,nlevs)
      REAL t_data(nazimuth,nrange,nlevs), r_data(nazimuth,nrange,nlevs)
      REAL lato, lono, startaz, endaz, range

      INTEGER i, j, z
      REAL s, s1, s2, r, s_inc, r_inc
      
      REAL cos_lato, sin_lato, cos_bearo, sin_bearo, 
     &     cos_dist, sin_dist, ux, uy

      REAL, PARAMETER :: radpdeg = 0.017453293

c  Here we set the data at center of the cylinder (r=0) to zero

      DO z=1,nlevs
        DO i=1,nazimuth
          u_data(1,1,z) = 0.0
        ENDDO
      ENDDO

c  Add a multiple of 360 to the ending azimuth to make it greater than 
c  the starting azimuth (this makes for easier looping)

      s1 = startaz
      s2 = endaz
      DO WHILE (s2 <= s1)
        s2 = s2 + 360.0
      ENDDO

c  Calculate increments of radius(r) and azimuth(s)

      r_inc = range/(nrange-1)
      s_inc = (s2-s1)/(nazimuth-1)

c  Precalculate sins and cosines of unvarying variables so we don't
c  have to in the inner loops

      cos_lato = cos(lato*radpdeg)
      sin_lato = sin(lato*radpdeg)

c  Below is the loop that calculates the tangential and radial 
c  components for r > 0

      r = r_inc

      DO i=2,nrange

        cos_dist = cos(r*radpdeg)
        sin_dist = sin(r*radpdeg)

        s = s1

        DO j=1,nazimuth
        
          cos_bearo = cos(s*radpdeg)
          sin_bearo = sin(s*radpdeg)

c Calculate the x and y components of the unit projection vector 
c pointing from the center point he geodesic

          CALL get_pvect(cos_lato,sin_lato,cos_dist,sin_dist,
     &                   cos_bearo,sin_bearo,ux,uy)

c Calculate the radial and tangential wind compononts

          DO z=1,nlevs
            r_data(j,i,z) = ux * u_data(j,i,z) + uy * v_data(j,i,z)
            t_data(j,i,z) = -uy * u_data(j,i,z) + ux * v_data(j,i,z)
          ENDDO

           s = s + s_inc

        ENDDO

        r = r + r_inc

      ENDDO
      
      RETURN
      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      SUBROUTINE get_pvect(cos_lato,sin_lato,cos_dist,sin_dist,
     &                     cos_bearo,sin_bearo,ux,uy)

      IMPLICIT NONE

      REAL cos_lato, sin_lato, cos_dist, sin_dist,
     &      cos_bearo, sin_bearo, ux, uy

      REAL cos_latt, sin_latt

      sin_latt = sin_lato*cos_dist + cos_lato*sin_dist*cos_bearo
      IF (sin_latt > 1.0) THEN 
        sin_latt = 1.0 
      ENDIF
      IF (sin_latt < -1.0) THEN
        sin_latt = -1.0
      ENDIF

      cos_latt = sqrt(1 - sin_latt*sin_latt)
      
      uy = (sin_latt*cos_dist - sin_lato) / (cos_latt*sin_dist)
      IF (uy > 1.0) THEN 
        uy = 1.0 
      ENDIF
      IF (uy < -1.0) THEN 
        uy = -1.0 
      ENDIF

      IF (sin_bearo >= 0) THEN
        ux = sqrt(1 - uy*uy)
      ELSE
        ux = -sqrt(1 - uy*uy)
      ENDIF

      RETURN
      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      SUBROUTINE close_files(iunit_gr_out,iunit_gr_in,
     &                      iunit_gr_u,iunit_gr_v,
     &                      iunit_gr_t,iunit_gr_r)

c  Close all files opened in program

      IMPLICIT NONE

      INTEGER iunit_gr_out, iunit_gr_in, 
     &        iunit_gr_u, iunit_gr_v, iunit_gr_t, iunit_gr_r

      CLOSE(iunit_gr_out)
      CLOSE(iunit_gr_in)
      CLOSE(iunit_gr_u)
      CLOSE(iunit_gr_v)
      CLOSE(iunit_gr_t)
      CLOSE(iunit_gr_r)

      RETURN
      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
